package it.unisannio.ding.ids.wedroid.app.view

import androidx.test.espresso.Espresso.closeSoftKeyboard
import androidx.test.espresso.Espresso.onView
import androidx.test.espresso.action.ViewActions.*
import androidx.test.espresso.assertion.ViewAssertions.matches
import androidx.test.espresso.contrib.RecyclerViewActions
import androidx.test.espresso.matcher.ViewMatchers.*
import androidx.test.ext.junit.runners.AndroidJUnit4
import androidx.test.filters.LargeTest
import androidx.test.rule.ActivityTestRule
import it.unisannio.ding.ids.wedroid.app.R
import it.unisannio.ding.ids.wedroid.app.view.adapter.BoardsListAdapter
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith

@RunWith(AndroidJUnit4::class)
@LargeTest
class BoardsListsActivityTest {

    @get:Rule
    val activityRule = ActivityTestRule(BoardsListsActivity::class.java)

    @Test
    fun openBoard() {
        onView(withId(R.id.boardList))
            .perform(
                RecyclerViewActions.actionOnItemAtPosition<BoardsListAdapter.BoardViewHolder>(
                    0, click()
                )
            )

        onView(withId(R.id.getLists))
            .check(matches(isDisplayed()))
    }

    @Test
    fun openBoardLists() {
        onView(withId(R.id.boardList))
            .perform(
                RecyclerViewActions.actionOnItemAtPosition<BoardsListAdapter.BoardViewHolder>(
                    0, click()
                )
            )

        onView(withId(R.id.getLists))
            .perform(click())

        onView(withId(R.id.synchronize))
            .check(matches(isDisplayed()))
    }

    @Test
    fun addNewCard() {
        onView(withId(R.id.boardList))
            .perform(
                RecyclerViewActions.actionOnItemAtPosition<BoardsListAdapter.BoardViewHolder>(
                    0, click()
                )
            )

        onView(withId(R.id.getLists))
            .perform(click())

        onView(withId(R.id.buttonAddCard))
            .perform(click())

        onView(withId(R.id.edittext))
            .perform(typeText("card test"))
        closeSoftKeyboard()
        onView(withText("OK"))
            .perform(click())

        onView(withId(R.id.synchronize))
            .check(matches(isDisplayed()))
    }

    @Test
    fun synchronizeBoard() {
        onView(withId(R.id.boardList))
            .perform(
                RecyclerViewActions.actionOnItemAtPosition<BoardsListAdapter.BoardViewHolder>(
                    1, click()
                )
            )

        onView(withId(R.id.getLists))
            .perform(click())

        onView(withId(R.id.synchronize))
            .perform(click())
    }

    @Test
    fun pullToRefresh() {
        onView(withId(R.id.pullToRefresh))
            .perform(swipeDown())
    }

    @Test
    fun openNewBoardActivity() {
        onView(withId(R.id.fab))
            .perform(click())

        onView(withId(R.id.newBoardName))
            .check(matches(isDisplayed()))
    }

    @Test
    fun newPrivateBoard() {
        onView(withId(R.id.fab))
            .perform(click())

        onView(withId(R.id.newBoardName))
            .perform(typeText("test private-board"))
        closeSoftKeyboard()
        onView(withId(R.id.newBoardDone))
            .perform(click())

        onView(withId(R.id.fab))
            .check(matches(isDisplayed()))
    }

    @Test
    fun newPublicBoard() {
        onView(withId(R.id.fab))
            .perform(click())

        onView(withId(R.id.newBoardName))
            .perform(typeText("test public-board"))
        closeSoftKeyboard()
        onView(withId(R.id.newBoardPermission))
            .perform(click())
        onView(withId(R.id.newBoardDone))
            .perform(click())

        onView(withId(R.id.fab))
            .check(matches(isDisplayed()))
    }

    @Test
    fun cancelNewBoard() {
        onView(withId(R.id.fab))
            .perform(click())

        onView(withId(R.id.newBoardCancel))
            .perform()
    }
}

